{-# LANGUAGE TypeApplications #-}


-- | Copyright  : Will Thompson and Iñaki García Etxebarria
-- License    : LGPL-2.1
-- Maintainer : Iñaki García Etxebarria
-- 
-- Data type containing a set of virtual methods used for
-- working on t'GI.HarfBuzz.Structs.FontT.FontT' font objects.
-- 
-- HarfBuzz provides a lightweight default function for each of
-- the methods in t'GI.HarfBuzz.Structs.FontFuncsT.FontFuncsT'. Client programs can implement
-- their own replacements for the individual font functions, as
-- needed, and replace the default by calling the setter for a
-- method.

#if (MIN_VERSION_haskell_gi_overloading(1,0,0) && !defined(__HADDOCK_VERSION__))
#define ENABLE_OVERLOADING
#endif

module GI.HarfBuzz.Structs.FontFuncsT
    ( 

-- * Exported types
    FontFuncsT(..)                          ,


 -- * Methods

#if defined(ENABLE_OVERLOADING)
    ResolveFontFuncsTMethod                 ,
#endif



    ) where

import Data.GI.Base.ShortPrelude
import qualified Data.GI.Base.ShortPrelude as SP
import qualified Data.GI.Base.Overloading as O
import qualified Prelude as P

import qualified Data.GI.Base.Attributes as GI.Attributes
import qualified Data.GI.Base.BasicTypes as B.Types
import qualified Data.GI.Base.ManagedPtr as B.ManagedPtr
import qualified Data.GI.Base.GArray as B.GArray
import qualified Data.GI.Base.GClosure as B.GClosure
import qualified Data.GI.Base.GError as B.GError
import qualified Data.GI.Base.GHashTable as B.GHT
import qualified Data.GI.Base.GVariant as B.GVariant
import qualified Data.GI.Base.GValue as B.GValue
import qualified Data.GI.Base.GParamSpec as B.GParamSpec
import qualified Data.GI.Base.CallStack as B.CallStack
import qualified Data.GI.Base.Properties as B.Properties
import qualified Data.GI.Base.Signals as B.Signals
import qualified Control.Monad.IO.Class as MIO
import qualified Data.Coerce as Coerce
import qualified Data.Text as T
import qualified Data.Kind as DK
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map
import qualified Foreign.Ptr as FP
import qualified GHC.OverloadedLabels as OL
import qualified GHC.Records as R
import qualified Data.Word as DW
import qualified Data.Int as DI
import qualified System.Posix.Types as SPT
import qualified Foreign.C.Types as FCT

-- Workaround for https://gitlab.haskell.org/ghc/ghc/-/issues/23392
#if MIN_VERSION_base(4,18,0)

#else

#endif

-- | Memory-managed wrapper type.
newtype FontFuncsT = FontFuncsT (SP.ManagedPtr FontFuncsT)
    deriving (Eq)

instance SP.ManagedPtrNewtype FontFuncsT where
    toManagedPtr (FontFuncsT p) = p

foreign import ccall "hb_gobject_font_funcs_get_type" c_hb_gobject_font_funcs_get_type :: 
    IO GType

type instance O.ParentTypes FontFuncsT = '[]
instance O.HasParentTypes FontFuncsT

instance B.Types.TypedObject FontFuncsT where
    glibType = c_hb_gobject_font_funcs_get_type

instance B.Types.GBoxed FontFuncsT

-- | Convert t'FontFuncsT' to and from 'Data.GI.Base.GValue.GValue'. See 'Data.GI.Base.GValue.toGValue' and 'Data.GI.Base.GValue.fromGValue'.
instance B.GValue.IsGValue (Maybe FontFuncsT) where
    gvalueGType_ = c_hb_gobject_font_funcs_get_type
    gvalueSet_ gv P.Nothing = B.GValue.set_boxed gv (FP.nullPtr :: FP.Ptr FontFuncsT)
    gvalueSet_ gv (P.Just obj) = B.ManagedPtr.withManagedPtr obj (B.GValue.set_boxed gv)
    gvalueGet_ gv = do
        ptr <- B.GValue.get_boxed gv :: IO (Ptr FontFuncsT)
        if ptr /= FP.nullPtr
        then P.Just <$> B.ManagedPtr.newBoxed FontFuncsT ptr
        else return P.Nothing
        
    


#if defined(ENABLE_OVERLOADING)
instance O.HasAttributeList FontFuncsT
type instance O.AttributeList FontFuncsT = FontFuncsTAttributeList
type FontFuncsTAttributeList = ('[ ] :: [(Symbol, DK.Type)])
#endif

#if defined(ENABLE_OVERLOADING)
type family ResolveFontFuncsTMethod (t :: Symbol) (o :: DK.Type) :: DK.Type where
    ResolveFontFuncsTMethod l o = O.MethodResolutionFailed l o

instance (info ~ ResolveFontFuncsTMethod t FontFuncsT, O.OverloadedMethod info FontFuncsT p) => OL.IsLabel t (FontFuncsT -> p) where
#if MIN_VERSION_base(4,10,0)
    fromLabel = O.overloadedMethod @info
#else
    fromLabel _ = O.overloadedMethod @info
#endif

#if MIN_VERSION_base(4,13,0)
instance (info ~ ResolveFontFuncsTMethod t FontFuncsT, O.OverloadedMethod info FontFuncsT p, R.HasField t FontFuncsT p) => R.HasField t FontFuncsT p where
    getField = O.overloadedMethod @info

#endif

instance (info ~ ResolveFontFuncsTMethod t FontFuncsT, O.OverloadedMethodInfo info FontFuncsT) => OL.IsLabel t (O.MethodProxy info FontFuncsT) where
#if MIN_VERSION_base(4,10,0)
    fromLabel = O.MethodProxy
#else
    fromLabel _ = O.MethodProxy
#endif

#endif


